package cz.muni.pa165.account.rest;

import cz.muni.pa165.account.facade.AccountFacade;
import cz.muni.pa165.core.api.account.AccountDto;
import io.swagger.v3.oas.annotations.Operation;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(path = "${rest.path.accounts}")
public class AccountRestController {

    private final AccountFacade accountFacade;

    @Autowired
    public AccountRestController(AccountFacade accountFacade) {
        this.accountFacade = accountFacade;
    }

    @Operation(summary = "Get account by ID")
    @GetMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<AccountDto> findById(@PathVariable Long id) {
        return ResponseEntity.ok(accountFacade.findById(id));
    }

    @Operation(summary = "Get all accounts")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public Page<AccountDto> findAll(@ParameterObject Pageable pageable) {
        return accountFacade.findAll(pageable);
    }

    @Operation(summary = "Create an account")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public AccountDto createAccount(@RequestBody AccountDto account) {
        return accountFacade.create(account);
    }

    @Operation(summary = "Update an account by ID")
    @PutMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.OK)
    public AccountDto updateAccount(
        @PathVariable Long id,
        @RequestBody AccountDto account
    ) {
        return accountFacade.update(id, account);
    }

    @Operation(summary = "Delete account by ID")
    @DeleteMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void deleteById(@PathVariable Long id) {
        accountFacade.deleteById(id);
    }
}
